local L = Capitalist.Locale
local realm = GetRealmName()
local character = UnitName("player")

local productsMemory = { }

function Capitalist:ProductMemoryReset()
	productsMemory = { }
end

function Capitalist:ProductIsEnabled(profession, skillName)
	return (self.db.profile.professions and
			self.db.profile.professions[profession] and
			self.db.profile.professions[profession][skillName] and
			(not self.db.profile.professions[profession][skillName].disabled))
end

function Capitalist:ProductSetEnabled(profession, skillName, enabled)
	if (enabled) then
		self.db.profile.professions[profession][skillName].disabled = nil
	else
		self.db.profile.professions[profession][skillName].disabled = true
	end
end

function Capitalist:ProductComponentsAreVisible(profession, skillName)
	return (not productsMemory[profession]) or
		   (not productsMemory[profession][skillName]) or
		   (not productsMemory[profession][skillName].hiddenComponents)
end

function Capitalist:ProductComponentsSetVisible(profession, skillName, visible)
	if (visible) then
		if (productsMemory[profession]) then
			if (productsMemory[profession][skillName]) then
				productsMemory[profession][skillName].hiddenComponents = nil
			end
		end
	else
		if (not productsMemory[profession]) then
			productsMemory[profession] = {}
		end
		if (not productsMemory[profession][skillName]) then
			productsMemory[profession][skillName] = {}
		end
		productsMemory[profession][skillName].hiddenComponents = true
	end
end

function Capitalist:ProductIsVisible(profession, skillName)
	return (not productsMemory[profession]) or
		   (not productsMemory[profession][skillName]) or
		   (not productsMemory[profession][skillName].hidden)
end

function Capitalist:ProductSetVisible(profession, skillName, visible)
	if (visible) then
		if (productsMemory[profession]) then
			if (productsMemory[profession][skillName]) then
				productsMemory[profession][skillName].hidden = nil
			end
		end
	else
		if (not productsMemory[profession]) then
			productsMemory[profession] = {}
		end
		if (not productsMemory[profession][skillName]) then
			productsMemory[profession][skillName] = {}
		end
		productsMemory[profession][skillName].hidden = true
	end
end

function Capitalist:FindProfession(skillName)
	if (self.db.profile.professions) then
		for profession, professionData in pairs(self.db.profile.professions) do
			if (professionData[skillName]) then
				return profession
			end
		end
	end
	return nil	-- explicit for clarity
end

function Capitalist:FindCrafter(profession, skillName)
	if (self.db.profile[realm]) then
		for crafter, crafterData in pairs(self.db.profile[realm]) do
			if (crafterData[profession] and
				crafterData[profession][skillName]) then
				return crafter
			end
		end
	end
	return nil	-- explicit for clarity
end

local function StringForItem(prefix, item, count)
	local s = item;
	if (prefix) then
		s = prefix..s
	end
	if (count and (count > 1)) then
		s = s.." ["..count.."]"
	end
	return s
end

local function ColorizeForSkillType(input, skillType)
	if (skillType == "trivial") then
		return "|cff808080"..input.."|r"
	elseif (skillType == "easy") then
		return "|cff00ff00"..input.."|r"
	elseif (skillType == "medium") then
		return "|cffffff00"..input.."|r"
	elseif (skillType == "optimal") then
		return "|cffff8000"..input.."|r"
	elseif (skillType == "difficult") then
		return "|cffff8080"..input.."|r"
	else
		return input
	end
end

function Capitalist:OutputProductLink(profession, prefix, skillName, count, crafter)
	if (self.db.profile.professions and
		self.db.profile.professions[profession] and
		self.db.profile.professions[profession][skillName]) then

		local s = StringForItem(prefix, self.db.profile.professions[profession][skillName].link, count)
		if (self.db.profile.professions[profession][skillName].min) then
			s = s.." ("..self.db.profile.professions[profession][skillName].min
			if (self.db.profile.professions[profession][skillName].max) then
				s = s.."-"..self.db.profile.professions[profession][skillName].max
			end
			s = s..")"
		end
		if (crafter) then
			local skillType = self.db.profile[realm][crafter][profession][skillName].type
			s = s.." by "..ColorizeForSkillType(crafter, skillType)
		end
		s = s.." with "..profession
		self:Print(s)
	end
end

function Capitalist:FindProducts(...)
	local pattern = ...
	if (self.db.profile.professions) then
		for profession, professionData in pairs(self.db.profile.professions) do
			for skillName, skillData in pairs(professionData) do
				if (string.find(strlower(skillName), strlower(pattern))) then
					self:OutputProductLink(profession, "", skillName, 1, crafter)
				end
			end
		end
	end
end

function Capitalist:OutputProductCraftingSequence(...)
	local productName, count, prefix = ...
	if (not count) then
		count = 1
	end
	if (not prefix) then
		prefix = ""
	end

	local skillName = productName
	if (L.isBar(productName)) then
		skillName = L.getSkillFromBar(productName)
	end

	local profession = self:FindProfession(skillName)
	local crafter = nil
	if (profession) then
		crafter = self:FindCrafter(profession, skillName)
		self:OutputProductLink(profession, prefix, skillName, count, crafter)
	else
		self:Print(StringForItem(prefix, productName, count))
	end

	if (crafter) then
		for componentName, componentCount in pairs(self.db.profile.professions[profession][skillName].components) do
			self:OutputProductCraftingSequence(componentName, componentCount, prefix.."    ")
		end
	end
end
